@@ -26,6 +26,8 @@ import logging
 import os
 from . import log_filter
 
+from vdsm.client import ServerError
+
 
 logger = logging.getLogger(__name__)
 
@@ -50,12 +52,6 @@ class StorageServer(object):
         self._user = self._config.get(config.ENGINE, config.ISCSI_USER)
         self._password = self._config.get(config.ENGINE, config.ISCSI_PASSWORD)
         self._port = self._config.get(config.ENGINE, config.ISCSI_PORT)
-        self._tpgt = None
-        try:
-            str_tpgt = self._config.get(config.ENGINE, config.ISCSI_TPGT)
-            self._tpgt = int(str_tpgt)
-        except (KeyError, ValueError):
-            pass
         self._mnt_options = None
         try:
             self._mnt_options = self._config.get(
@@ -64,6 +60,14 @@ class StorageServer(object):
             )
         except (KeyError, ValueError):
             pass
+        self._nfs_version = None
+        try:
+            self._nfs_version = self._config.get(
+                config.ENGINE,
+                config.NFS_VERSION
+            )
+        except (KeyError, ValueError):
+            pass
 
     def _get_conlist_nfs_gluster(self):
         conDict = {
@@ -71,12 +75,22 @@ class StorageServer(object):
             'user': 'kvm',
             'id': self._connectionUUID,
         }
-        if self._domain_type == constants.DOMAIN_TYPE_NFS3:
+        if self._domain_type == constants.DOMAIN_TYPE_NFS:
             storageType = constants.STORAGE_TYPE_NFS
-            conDict['protocol_version'] = 3
+            if not self._nfs_version:
+                conDict['protocol_version'] = 'auto'
+            else:
+                conDict['protocol_version'] = self._nfs_version.replace(
+                    'v', ''
+                ).replace(
+                    '_', '.'
+                )
+        elif self._domain_type == constants.DOMAIN_TYPE_NFS3:
+            storageType = constants.STORAGE_TYPE_NFS
+            conDict['protocol_version'] = '3'
         elif self._domain_type == constants.DOMAIN_TYPE_NFS4:
             storageType = constants.STORAGE_TYPE_NFS
-            conDict['protocol_version'] = 4
+            conDict['protocol_version'] = '4'
         elif self._domain_type == constants.DOMAIN_TYPE_GLUSTERFS:
             storageType = constants.STORAGE_TYPE_GLUSTERFS
             conDict['vfs_type'] = 'glusterfs'
@@ -110,53 +124,60 @@ class StorageServer(object):
         if they differ, raises
         hosted_engine.DuplicateStorageConnectionException
         See rhbz#1300749
-        :param cli:  a jsonrpcvdscli instance
+        :param cli:  a vdsm.client instance
         :param path: the path (without the trailing '/') to be validated
                      against already connected storage server
         :raise       hosted_engine.DuplicateStorageConnectionException on error
                      to prevent connecting twice the hosted-engine storage
                      server
         """
-        response = cli.getStorageDomainInfo(self._sdUUID)
-        if response['status']['code'] != 0:
+        try:
+            cli.StorageDomain.getInfo(storagedomainID=self._sdUUID)
+        except ServerError:
             self._log.debug(
                 'Storage domain {sd} is not available'.format(sd=self._sdUUID)
             )
-        else:
-            # verifying only if the storage domain is already connected
-            canonical_path = env_path.canonize_file_path(
-                self._domain_type,
-                path,
-                self._sdUUID,
+            return
+
+        # verifying only if the storage domain is already connected
+        canonical_path = env_path.canonize_file_path(
+            self._domain_type,
+            path,
+            self._sdUUID,
+        )
+        effective_path = env_path.get_domain_path(self._config)
+        if effective_path != canonical_path:
+            msg = (
+                "The hosted-engine storage domain is already "
+                "mounted on '{effective_path}' with a path that is "
+                "not supported anymore: the right path should be "
+                "'{canonical_path}'."
+            ).format(
+                canonical_path=canonical_path,
+                effective_path=effective_path,
             )
-            effective_path = env_path.get_domain_path(self._config)
-            if effective_path != canonical_path:
-                msg = (
-                    "The hosted-engine storage domain is already "
-                    "mounted on '{effective_path}' with a path that is "
-                    "not supported anymore: the right path should be "
-                    "'{canonical_path}'."
-                ).format(
-                    canonical_path=canonical_path,
-                    effective_path=effective_path,
-                )
-                self._log.error(msg)
-                raise ex.DuplicateStorageConnectionException(msg)
+            self._log.error(msg)
+            raise ex.DuplicateStorageConnectionException(msg)
 
     def _get_conlist_iscsi(self):
         storageType = constants.STORAGE_TYPE_ISCSI
-        conDict = {
-            'connection': self._storage,
-            'iqn': self._iqn,
-            'portal': self._portal,
-            'user': self._user,
-            'password': self._password,
-            'id': self._connectionUUID,
-            'port': self._port,
-        }
-        if self._tpgt:
-            conDict['tpgt'] = self._tpgt
-        conList = [conDict]
+        conList = []
+        ip_port_list = [
+            {'ip': x[0], 'port': x[1]} for x in zip(
+                self._storage.split(','),
+                self._port.split(',')
+            )
+        ]
+        for x in ip_port_list:
+            conList.append({
+                'connection': x['ip'],
+                'iqn': self._iqn,
+                'tpgt': self._portal,
+                'user': self._user,
+                'password': self._password,
+                'id': self._connectionUUID,
+                'port': x['port'],
+            })
         return conList, storageType
 
     def _get_conlist_fc(self):
@@ -164,20 +185,6 @@ class StorageServer(object):
         conList = []
         return conList, storageType
 
-    def _check_connection(self, status):
-        self._log.debug(status)
-        if status['status']['code'] != 0:
-            raise RuntimeError(
-                'Connection to storage server failed: %s' %
-                status['status']['message']
-            )
-        cl = status['items'] if 'items' in status else []
-        for con in cl:
-            if con['status'] != 0:
-                raise RuntimeError(
-                    'Connection to storage server failed'
-                )
-
     def _get_conlist(self, cli, normalize_path):
         """
         helper method to get conList parameter for connectStorageServer and
@@ -188,6 +195,7 @@ class StorageServer(object):
         conList = None
         storageType = None
         if self._domain_type in (
+                constants.DOMAIN_TYPE_NFS,
                 constants.DOMAIN_TYPE_NFS3,
                 constants.DOMAIN_TYPE_NFS4,
                 constants.DOMAIN_TYPE_GLUSTERFS,
@@ -222,7 +230,12 @@ class StorageServer(object):
             logger=self._log,
             timeout=constants.VDSCLI_SSL_TIMEOUT
         )
-        status = cli.repoStats(domains=[self._sdUUID])
+        try:
+            status = cli.Host.getStorageRepoStats(domains=[self._sdUUID])
+        except ServerError as e:
+            self._log.error(str(e))
+            return False
+
         try:
             valid = status[self._sdUUID]['valid']
             delay = float(status[self._sdUUID]['delay'])
@@ -244,17 +257,33 @@ class StorageServer(object):
         conList, storageType = self._get_conlist(cli, normalize_path=True)
         if conList:
             self._log.info("Connecting storage server")
-            status = cli.connectStorageServer(
-                storagepoolID=self._spUUID,
-                domainType=storageType,
-                connectionParams=conList,
-            )
-            self._check_connection(status)
+            try:
+                connections = cli.StoragePool.connectStorageServer(
+                    storagepoolID=self._spUUID,
+                    domainType=storageType,
+                    connectionParams=conList,
+                )
+                self._log.debug(connections)
+            except ServerError as e:
+                raise RuntimeError(
+                    'Connection to storage server failed: %s' %
+                    str(e)
+                )
+
+            for con in connections:
+                if con['status'] != 0:
+                    raise RuntimeError(
+                        'Connection to storage server failed'
+                    )
+
         self._log.info("Refreshing the storage domain")
         # calling getStorageDomainStats has the side effect of
         # causing a Storage Domain refresh including
         # all its tree under /rhev/data-center/...
-        cli.getStorageDomainStats(self._sdUUID)
+        try:
+            cli.StorageDomain.getStats(storagedomainID=self._sdUUID)
+        except ServerError as e:
+            self._log.debug("Error refreshing storage domain: %s", str(e))
 
     def disconnect_storage_server(self):
         """
@@ -269,19 +298,20 @@ class StorageServer(object):
         # from where we were connected also if its path was wrong
         conList, storageType = self._get_conlist(cli, normalize_path=False)
         if conList:
-            status = cli.disconnectStorageServer(
-                storagepoolID=self._spUUID,
-                domainType=storageType,
-                connectionParams=conList,
-            )
-            self._log.debug(status)
-            if status['status']['code'] != 0:
+            try:
+                status = cli.StoragePool.disconnectStorageServer(
+                    storagepoolID=self._spUUID,
+                    domainType=storageType,
+                    connectionParams=conList,
+                )
+                self._log.debug(status)
+            except ServerError as e:
                 raise RuntimeError(
                     (
                         'Disconnection to storage server failed, unable '
                         'to recover: {message} - Please try rebooting the '
                         'host to reach a consistent status'
                     ).format(
-                        message=status['status']['message']
+                        message=str(e)
                     )
                 )
